/*---------------------------------------------------------------------------*
 |    The "Aladdin Free Public License" doesn't require any license fee,     |
 |    and allows you to                                                      |
 |    - develop and distribute PDFlib-based software for which the complete  |
 |      source code is made available                                        |
 |    - redistribute PDFlib non-commercially under certain conditions        |
 |    - redistribute PDFlib on digital media for a fee if the complete       |
 |      contents of the media are freely redistributable                     |
 |    Details can be found in the file aladdin-license.pdf.                  |
 |                                                                           |
 |    These conditions extend to ports to other programming languages.       |
 |    PDFlib is distributed with no warranty of any kind. Commercial users,  |
 |    however, will receive warranty and support statements in writing.      |
 *---------------------------------------------------------------------------*/

/* evschei2.c
 *
 * Entlassungs-/Verlegungschein generieren
 * Stefan Langenberg / HNO-Klinik / Uniklinikum Bonn
 *
 * Diese Version kommt mit beliebig vielen Diagnosen und Prozeduren zurecht. 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <dos.h>

#include "pdflib.h"

#define P_MAX 1000
#define Z_MAX 20
#define B_MAX 1000  

typedef struct {
    char    evgrund[B_MAX];
    char    evziel[B_MAX];
    char    evdatum[B_MAX];
    char    evzeit[B_MAX];
    char    stammdaten[B_MAX];
    char    pdatum[B_MAX];
    char    pabt[B_MAX];
    char    fahl[B_MAX];
    char    se[B_MAX];
    char    fp[B_MAX];
    char    khd[B_MAX];
    char    beatmung[B_MAX];
    char    gewicht[B_MAX];
    char    arzt[B_MAX];
} evdata;  

evdata d;

struct date heute;
char today[11];

int intern=0;

float scale_x, scale_y;
float offset_x, offset_y;
PDF *p;

int image;
int font;
    
float xpos,ypos;
float width, height;

static char diagnose[P_MAX][Z_MAX];
static char prozedur[P_MAX][Z_MAX];
static char fahl[P_MAX][Z_MAX];
static char pdatum[P_MAX][Z_MAX];
static char pabt[P_MAX][Z_MAX];

static int max = 0;
static int maxdiag = 0;
static int maxproz = 0;
static int nrun = 0;
static int offset = 0;


int
rerror(int ln)
{
    fprintf(stderr, "Fehler(4): Beim Lesen der Parameterdatei in Zeile %d\n",ln);
    exit(4);
}    


/* Ldt Parameterdatei in Struktur evdata
 *
 */

int
loaddata(const char *fname)
{
    char        buffer[10000];
    char        *s;
    FILE        *fptr;
    #ifdef _DEBUG
    int         i;
    #endif
            
    fptr = fopen(fname,"r");
    if ( fptr == NULL ) return -1;
    else if ( fgets(buffer,sizeof(buffer),fptr) == NULL ) rerror(1);
    else {
        strncpy(d.evgrund,buffer,sizeof(d.evgrund));
        #ifdef _DEBUG
        printf("%s",d.evgrund);
        #endif
    }    
    if ( fgets(buffer,sizeof(buffer),fptr) == NULL ) rerror(2);
    else {
        strncpy(d.evziel,buffer,sizeof(d.evziel));
        #ifdef _DEBUG
        printf("%s",d.evziel);
        #endif
    }
    if ( fgets(buffer,sizeof(buffer),fptr) == NULL ) rerror(3);
    else {
        strncpy(d.evdatum,buffer,sizeof(d.evdatum));
        #ifdef _DEBUG
        printf("%s",d.evdatum);
        #endif
    }
    if ( fgets(buffer,sizeof(buffer),fptr) == NULL ) rerror(4);
    else {
        strncpy(d.evzeit,buffer,sizeof(d.evzeit));
        #ifdef _DEBUG
        printf("%s",d.evzeit);
        #endif
    }
    if ( fgets(buffer,sizeof(buffer),fptr) == NULL ) rerror(5);
    else {
        strncpy(d.stammdaten,buffer,sizeof(d.stammdaten));
        #ifdef _DEBUG
        printf("%s",d.stammdaten);
        #endif
    }
    if ( fgets(buffer,sizeof(buffer),fptr) == NULL ) rerror(6);
    else {
        max = 0;
        s = strtok(buffer,"|");
        if (s != NULL) {
            strncpy(diagnose[max],s,Z_MAX);
            max++;
            while( (s=strtok(NULL,"|")) != NULL )
            {
                strncpy(diagnose[max],s,Z_MAX);
                max++;
            }
        }
        maxdiag = max - 1;    
        #ifdef _DEBUG
        printf("%d Diagnose(n):\n",maxdiag);
        for (i=0;i<maxdiag;i++)
        {
            printf("%d\t",i);
            printf("%s\n",diagnose[i]);
        }
        #endif
    }
    if ( fgets(buffer,sizeof(buffer),fptr) == NULL ) rerror(7);
    else {
        max = 0;
        s = strtok(buffer,"|");
        if (s != NULL) {
            strncpy(prozedur[max],s,Z_MAX);
            max++;
            while( (s=strtok(NULL,"|")) != NULL )
            {
                strncpy(prozedur[max],s,Z_MAX);
                max++;
            }
        }    
        maxproz = max - 1;    
        #ifdef _DEBUG
        printf("%d Prozedur(en):\n",maxproz);
        for (i=0;i<maxproz;i++)
        {
            printf("%d\t",i);
            printf("%s\n",prozedur[i]);
        }    
        #endif
    }
    if ( fgets(buffer,sizeof(buffer),fptr) == NULL ) rerror(8);
    else {
        max = 0;
        s = strtok(buffer,"|");
        if (s != NULL) {
            strncpy(pdatum[max],s,Z_MAX);
            max++;
            while( (s=strtok(NULL,"|")) != NULL )
            {
                strncpy(pdatum[max],s,Z_MAX);
                max++;
            }
        }    
        #ifdef _DEBUG
        for (i=0;i<maxproz;i++)
        {    
            printf("%d\t",i);
            printf("%s\n",pdatum[i]);
        }
        #endif 
    }
    if ( fgets(buffer,sizeof(buffer),fptr) == NULL ) rerror(9);
    else {
        max = 0;
        s = strtok(buffer,"|");
        if (s != NULL) {
            strncpy(pabt[max],s,Z_MAX);
            max++;
            while( (s=strtok(NULL,"|")) != NULL )
            {
                strncpy(pabt[max],s,Z_MAX);
                max++;
            }
        }     
        #ifdef _DEBUG
        for (i=0;i<maxproz;i++)
        {
            printf("%d\t",i);
            printf("%s\n",pabt[i]);
        }    
        #endif   
    }
    if ( fgets(buffer,sizeof(buffer),fptr) == NULL ) rerror(10);
    else {
        max = 0;
        s = strtok(buffer,"|");
        if (s != NULL) {
            strncpy(fahl[max],s,Z_MAX);
            max++;        
            while( (s=strtok(NULL,"|")) != NULL )
            {
                strncpy(fahl[max],s,Z_MAX);
                max++;
            }
        }       
        #ifdef _DEBUG
        for (i=0;i<maxproz;i++)
        {
            printf("%d\t",i);
            printf("%s\n",fahl[i]);
        }
        #endif
    }
    if ( fgets(buffer,sizeof(buffer),fptr) == NULL ) rerror(11);
    else {
        strncpy(d.se,buffer,sizeof(d.se));
        #ifdef _DEBUG
        printf("%s",d.se);
        #endif
    }
    if ( fgets(buffer,sizeof(buffer),fptr) == NULL ) rerror(12);
    else {
        strncpy(d.fp,buffer,sizeof(d.fp));
        #ifdef _DEBUG
        printf("%s",d.fp);
        #endif  
    }
    if ( fgets(buffer,sizeof(buffer),fptr) == NULL ) rerror(13);
    else {
        strncpy(d.khd,buffer,sizeof(d.khd));
        #ifdef _DEBUG
        printf("%s",d.khd);
        #endif   
    }
    if ( fgets(buffer,sizeof(buffer),fptr) == NULL ) rerror(14);
    else {    
        strncpy(d.beatmung,buffer,sizeof(d.beatmung));
        #ifdef _DEBUG
        printf("%s",d.beatmung);
        #endif
    }
    if (fgets(buffer,sizeof(buffer),fptr) == NULL ) rerror(15);
    else {
        strncpy(d.gewicht,buffer,sizeof(d.gewicht));
        #ifdef _DEBUG
        printf("%s",d.gewicht);
        #endif
    }
    if ( fgets(buffer,sizeof(buffer),fptr) == NULL ) rerror(16);
    strncpy(d.arzt,buffer,sizeof(d.arzt));
    #ifdef _DEBUG
    printf("%s",d.arzt);
    #endif 
    fclose(fptr);
    max = (maxdiag > maxproz) ? maxdiag : maxproz;   
    return 0;    
}
 

/* Gibt Text an der Position x,y aus, durch | wird ein
 * Zeilenvorschub ausgelst
 */

int
PDF_printf(float x, float y, char *text)
{
    char *s;
    float xx,yy;
    char buffer[10000];
        
    strcpy(buffer,text);
    /* Koordinaten umrechnen */
    xx = (x - offset_x) * 28.346;
    yy = (y - offset_y) * 28.346; 
    s = strtok(buffer,"|");
    if (s != NULL) 
    {
        PDF_set_text_pos(p,xx,yy);
        PDF_show(p,s);
        while( (s=strtok(NULL,"|")) != 0 )
            PDF_continue_text(p,s);    
        return 0;                   
    } else return -1;
}                

/* Gibt Array ab der Position x,y vertikal aus
 */

int
PDF_array_printf(float x, float y, char array[P_MAX][Z_MAX], int adim)
{
    float xx,yy;
    int i;
                 
    if (adim < 1 ) return -1;
    xx = (x - offset_x) * 28.346;
    yy = (y - offset_y) * 28.346; 
    PDF_set_text_pos(p,xx,yy);
    PDF_show(p,array[offset]); 
    for (i=1;i<15;i++) {
        if ( (offset + i) < adim ) {
            PDF_continue_text(p, array[i+offset]);
        }    
    }            
    return 0;                   
}                


/* Rechteck in cm zeichnen */
void PDF_rectcm(float x, float y, float width, float height)
{
    x = (x - offset_x) * 28.346;
    y = (y - offset_y) * 28.346;
    width = width * 28.346;
    height = height * 28.346;
    PDF_rect(p,x,y,width,height);  
}


/*  Informationen ausgeben
 *
 */

void
display(void)
{  
    /* We generate a page with the image's dimensions */    
    
    PDF_save(p);
    PDF_scale(p, scale_x, scale_y);
    PDF_place_image(p, image, (float) xpos/scale_x, (float) ypos/scale_y, (float) 1.0);
    PDF_restore(p);    
    
    PDF_setfont(p, font, 10);    
    /* Entlassung-/Verlegung */
    PDF_printf(6.0, 21.3, d.evgrund); 
    if ( strncmp(d.evgrund,"06",2) == 0 ) {
        PDF_printf(2.1, 23.6, "X");
        PDF_printf(3.0, 23.1, d.evziel);
    } else if ( strncmp(d.evgrund,"12",2) == 0 ) {
        PDF_printf(2.1, 24.2, "X");
    } else PDF_printf(2.1, 24.9, "X");
    PDF_printf(5.5, 22.1, d.evdatum);
    PDF_printf(10.0, 22.1, d.evzeit);
    PDF_printf(13.0, 26.3, d.stammdaten);             
    PDF_printf(4.0, 6.0, d.khd);
    PDF_printf(3.1, 3.3, today);
    PDF_printf(6.6, 3.3, d.arzt);
        
    PDF_save(p);
    PDF_set_value(p, "leading", 24);    
    PDF_array_printf(4.0, 19.1, diagnose, maxdiag);
    PDF_array_printf(7.5, 19.1, prozedur, maxproz);
    PDF_array_printf(10.9, 19.1, pdatum, maxproz);
    PDF_array_printf(13.1, 19.1, pabt, maxproz);
    PDF_printf(17.5, 4.1, d.gewicht);
    if ( strncmp(d.evgrund,"02",2) == 0 )
        PDF_printf(14.9, 3.2, "X");  
    if (intern == 1) {
        PDF_array_printf(14.8, 19.1, fahl, maxproz);
        PDF_printf(15.5, 19.1, d.se);
        PDF_printf(18.0, 19.1, d.fp);
        PDF_printf(17.5, 5.5, d.beatmung);
    }
    PDF_restore(p);
}    
  
        



int
main(int argc, char **argv)
{
    char stitel[100];
    int j=0;
    
    if (argc != 3) {
        printf("Verwendung: evschein <Parameterdatei> <PDF-Datei>\n");
        exit(0);
    }
    
            
    /* Parameterdatei einlesen */
    if (loaddata(argv[1]) == -1) {
        fprintf(stderr, "Fehler(1): beim ffnen der Parameterdatei %s\n",argv[1]);
        exit(1);
    }   
        
          
    p = PDF_new();

        
    /* open new PDF file */
    if (PDF_open_file(p, argv[2]) == -1) {
	    fprintf(stderr, "Fehler(2): %s kann nicht erzeugt werden\n",argv[2]);
	    exit(2);
    }
    
    PDF_set_info(p, "Creator", "HNO-Tumordukumentation");
    PDF_set_info(p, "Author", "Stefan Langenberg");
    PDF_set_info(p, "Title", "Entlassungs-/Verlegungsschein");
    PDF_set_parameter(p, "openmode", "bookmarks");
       
    image = PDF_open_image_file(p, "tiff", "./evschein.tif", "", 0);
    
    if (image == -1) {
	    fprintf(stderr, "Fehler(3): Kann Datei evschein.tif nicht lesen\n");
	    exit(3);
    }
    
    font = PDF_findfont(p, "Helvetica", "host", 0);
 
    /* Sollbreite des Formulars */
    width = 549; 
    height = 734;
    /* Skalierungsfaktoren */
    scale_x = 0.31;
    scale_y = 0.31;
    /* Offset */
    xpos = 23.0;
    ypos = 54.0;
    /* Korrektur */    
    offset_x = 0.0;
    offset_y = 0.0;
    
    getdate(&heute);
    sprintf(today,"%02d.%02d.%4d",heute.da_day,heute.da_mon,heute.da_year);
    
    /* Anzahl der Formulare */
    
    nrun = (int)floor( (max - 1.0) / 15.0 );
    if (max > P_MAX) {
        fprintf(stderr,"Fehler (4): So viele Daten knnen nicht verarbeitet werden!\n");
        exit(4);
    }    
    
           
    /* Formular Krankenkasse */
        
    for (j=0;j<=nrun;j++)
    {
        intern=0;
        offset = 15 * j;
        if (nrun > 0 ) {        
            sprintf(stitel,"Krankenkasse (%u/%u)", j + 1, nrun + 1);
        } else strcpy(stitel,"Krankenkasse");    
        
        PDF_begin_page(p, a4_width, a4_height);
        PDF_add_bookmark(p, stitel, 0, 0);
        display(); 
        PDF_setfont(p,font,14);
        PDF_printf(2.0, 2.0, stitel);
        PDF_setgray(p, 1.0);
        PDF_rectcm(14.7, 5.2, 5.5, 15.3); 
        PDF_fill(p);
        PDF_end_page(p);
    }
     
    /* Verwaltung */
    
    
    intern=1;
    for (j=0;j<=nrun;j++)
    {    
        offset = 15 * j;
        if (nrun > 0 ) {        
            sprintf(stitel,"Verwaltung (%u/%u)",j + 1,nrun + 1);
        } else strcpy(stitel,"Verwaltung"); 
        
        PDF_begin_page(p, a4_width, a4_height);
        PDF_add_bookmark(p, stitel, 0, 0);
        display(); 
        PDF_setfont(p,font,14);
        PDF_printf(2.0, 2.0, stitel);
        PDF_end_page(p);
    }
   
      
    for (j=0;j<=nrun;j++)
    {
        offset = 15 * j;
        if (nrun > 0 ) {        
            sprintf(stitel,"Patientenakte (%u/%u)",j + 1, nrun + 1);
        } else strcpy(stitel,"Patientenakte");
        
        PDF_begin_page(p, a4_width, a4_height);
        PDF_add_bookmark(p, stitel, 0, 0);
        display(); 
        PDF_setfont(p,font,14);
        PDF_printf(2.0, 2.0, stitel);
        PDF_end_page(p); 
    }
        
      
    /*
    for (j=0;j<=nrun;j++)
    {
        if (nrun > 0) {
            sprintf(stitel,"Klinik (%u/%u)",j + 1, nrun + 1);   
        } else strcpy(stitel,"Klinik");  
        offset = 15 * j;
        
        PDF_begin_page(p, a4_width, a4_height);
        PDF_add_bookmark(p, stitel, 0, 0);
        display();
        PDF_setfont(p,font,14);
        PDF_printf(2.0, 2.0, stitel);
        PDF_end_page(p); 
    }  
    */
    
          
    if ( strncmp(d.evgrund,"12",2) == 0 ) {
        if (nrun > 0 ) {        
            sprintf(stitel,"UKB interne Verlegung (%u/%u)",j + 1, nrun + 1);
        } else strcpy(stitel,"UKB interne Verlegung (Schein dem Patienten mitgeben)");      
        for (j=0;j<=nrun;j++)
        {
            offset = 15 * j;
            
            PDF_begin_page(p, a4_width, a4_height);
            PDF_add_bookmark(p, stitel, 0, 0);
            display();
            PDF_printf(2.0, 2.0, stitel);
            PDF_end_page(p);
        }        
    }
        
    PDF_close_image(p, image); 
  
    PDF_close(p);				/* close PDF document	*/

    PDF_delete(p);				/* delete the PDF object */

    exit(0);
}
